@@ -1,4 +1,6 @@
+import datetime
 import requests
+import time
 from .data import (
     _source,
     _headers,
@@ -14,6 +16,7 @@ from .data import (
     _vaccinations_timestamp_req,
     _vaccinations_age_group_by_region_on_day_requests,
     _vaccination_by_manufacturer_supplied_used_requests,
+    _vaccinations_gender_by_date_requests,
 )
 from .parser import (
     _parse_vaccinations_by_age,
@@ -28,11 +31,13 @@ from .parser import (
     _parse_vaccinations_timestamp,
     _parse_vaccinations_age_group_by_region_on_day,
     _parse_vaccinations_by_manufacturer_supplied_used,
+    _parse_vaccinations_gender_by_date,
 )
 
 from .types import (
     VaccinationByAgeRow,
     VaccinationByDayRow,
+    VaccinationsByGender,
     VaccineSupplyUsage,
     VaccinationByRegionRow,
     VaccinationByManufacturerRow,
@@ -41,7 +46,7 @@ from .types import (
     VaccinationAgeGroupByRegionOnDay,
 )
 
-from .enums import Manufacturer, Region, AgeGroup
+from .enums import Manufacturer, Region, AgeGroup, Gender
 
 
 def _get_data(req, parse_response):
@@ -87,42 +92,19 @@ def vaccines_supplied_by_manufacturer_cumulative() -> "list[VaccinationByManufac
 def vaccinations_by_age_group(
     group: AgeGroup = None,
 ) -> "dict[AgeGroup,list[VaccinationByDayRow]] or list[VaccinationByDayRow]":
-    def getBothDoses(dose1, dose2):
-        date = None
-        first_dose = None
-        second_dose = None
-        result = []
-        for dose in dose1:
-            date = dose.date
-            first_dose = dose.dose
-            second_doses = [dose for dose in dose2 if dose.date == date]
-            second_dose = second_doses[0].dose if len(second_doses) > 0 else None
-            result.append(
-                VaccinationByDayRow(
-                    date=date, first_dose=first_dose, second_dose=second_dose
-                )
-            )
-        return result
 
     obj = {}
     if group == None:
 
         key_value = _vaccination_by_age_group_requests.items()
         for key, req_list in key_value:
-            dose1_req = req_list[0]
-            dose2_req = req_list[1]
-            dose1 = _get_data(dose1_req, _parse_vaccinations_by_age_group)
-            dose2 = _get_data(dose2_req, _parse_vaccinations_by_age_group)
-            both_doses = getBothDoses(dose1, dose2)
-            obj[key] = both_doses
+            req = req_list[0]
+
+            obj[key] = _get_data(req, _parse_vaccinations_by_age_group)
         return obj
 
-    dose1_req = _vaccination_by_age_group_requests[group][0]
-    dose2_req = _vaccination_by_age_group_requests[group][1]
-    dose1 = _get_data(dose1_req, _parse_vaccinations_by_age_group)
-    dose2 = _get_data(dose2_req, _parse_vaccinations_by_age_group)
-    both_doses = getBothDoses(dose1, dose2)
-    return both_doses
+    req = _vaccination_by_age_group_requests[group][0]
+    return _get_data(req, _parse_vaccinations_by_age_group)
 
 
 # by region by day
@@ -190,3 +172,59 @@ def vaccinations_by_manufacturer_supplied_used(
     req = _vaccination_by_manufacturer_supplied_used_requests[group][0]
     doses = _get_data(req, _parse_vaccinations_by_manufacturer_supplied_used)
     return doses
+
+
+# PAGE 1
+# gender
+def vaccinations_gender_by_date(date: datetime.datetime = None):
+
+    start = time.perf_counter()
+
+    if date == None:
+        result = []
+        for day in _vaccinations_gender_by_date_requests:
+            date = day["date"]
+            female = day[Gender.FEMALE]
+            male = day[Gender.MALE]
+            female_first = _get_data(female[0], _parse_vaccinations_gender_by_date)
+            female_second = _get_data(female[1], _parse_vaccinations_gender_by_date)
+            male_first = _get_data(male[0], _parse_vaccinations_gender_by_date)
+            male_second = _get_data(male[1], _parse_vaccinations_gender_by_date)
+            result.append(
+                VaccinationsByGender(
+                    date=date,
+                    female_first=female_first,
+                    female_second=female_second,
+                    male_first=male_first,
+                    male_second=male_second,
+                )
+            )
+        finish = time.perf_counter()
+        print(f"Elapsed time: {finish - start}")
+        return result
+
+    filtered_days = list(
+        filter(lambda item: item["date"] == date, _vaccinations_gender_by_date_requests)
+    )
+
+    if len(filtered_days) == 0:
+        return None
+
+    day = filtered_days[0]
+
+    date = day["date"]
+    female = day[Gender.FEMALE]
+    male = day[Gender.MALE]
+    female_first = _get_data(female[0], _parse_vaccinations_gender_by_date)
+    female_second = _get_data(female[1], _parse_vaccinations_gender_by_date)
+    male_first = _get_data(male[0], _parse_vaccinations_gender_by_date)
+    male_second = _get_data(male[1], _parse_vaccinations_gender_by_date)
+    finish = time.perf_counter()
+    print(f"Elapsed time: {finish - start}")
+    return VaccinationsByGender(
+        date=date,
+        female_first=female_first,
+        female_second=female_second,
+        male_first=male_first,
+        male_second=male_second,
+    )
