@@ -29,13 +29,35 @@ def _parse_vaccinations_timestamp(data):
 
 
 def _parse_vaccinations_by_day(data) -> "list[VaccinationByDayRow]":
+    if "DS" not in data["results"][0]["result"]["data"]["dsr"]:
+        error = data["results"][0]["result"]["data"]["dsr"]["DataShapes"][0][
+            "odata.error"
+        ]
+        print(error)
+        raise Exception("Something went wrong!")
     resp = data["results"][0]["result"]["data"]["dsr"]["DS"][0]["PH"][0]["DM0"]
     parsed_data = []
 
+    date = None
+    people_vaccinated = None
+    people_fully_vaccinated = None
     for element in resp:
-        date = parse_date(element["G0"])
-        people_vaccinated = element["X"][0]["M0"]
-        people_fully_vaccinated = element["X"][1]["M0"] if len(element["X"]) > 1 else 0
+        C = element["C"]
+        if len(C) == 3:
+            date = parse_date(C[0])
+            people_vaccinated = C[1]
+            people_fully_vaccinated = C[2]
+        elif len(C) == 2:
+            date = parse_date(C[0])
+            R = element["R"]
+            if R == 2:
+                people_fully_vaccinated = C[1]
+            else:
+                people_vaccinated = C[1]
+        elif len(C) == 1:
+            date = parse_date(C[0])
+        else:
+            raise Exception("Unknown item length!")
 
         parsed_data.append(
             VaccinationByDayRow(
@@ -49,15 +71,22 @@ def _parse_vaccinations_by_day(data) -> "list[VaccinationByDayRow]":
 
 
 def _parse_vaccinations_by_age(data) -> "list[VaccinationByAgeRow]":
+    if "DS" not in data["results"][0]["result"]["data"]["dsr"]:
+        error = data["results"][0]["result"]["data"]["dsr"]["DataShapes"][0][
+            "odata.error"
+        ]
+        print(error)
+        raise Exception("Something went wrong!")
     resp = data["results"][0]["result"]["data"]["dsr"]["DS"][0]["PH"][0]["DM0"]
     parsed_data = []
 
     for element in resp:
-        age_group = str(element["G0"])
-        count_first = int(element["X"][0]["C"][1])
-        count_second = int(element["X"][1]["C"][1])
-        share_first = float(element["X"][0]["C"][0]) / 100.0
-        share_second = float(element["X"][1]["C"][0]) / 100.0
+        C = element["C"]
+        age_group = str(C[0])
+        share_second = float(C[1]) / 100.0
+        share_first = float(C[2]) / 100.0
+        count_first = int(C[3])
+        count_second = int(C[4])
 
         parsed_data.append(
             VaccinationByAgeRow(
@@ -73,11 +102,18 @@ def _parse_vaccinations_by_age(data) -> "list[VaccinationByAgeRow]":
 
 
 def _parse_vaccines_supplied_and_used(data) -> "list[VaccineSupplyUsage]":
+    if "DS" not in data["results"][0]["result"]["data"]["dsr"]:
+        error = data["results"][0]["result"]["data"]["dsr"]["DataShapes"][0][
+            "odata.error"
+        ]
+        print(error)
+        raise Exception("Something went wrong!")
+
     resp = data["results"][0]["result"]["data"]["dsr"]["DS"][0]["PH"][0]["DM0"]
     parsed_data = []
 
     for element in resp:
-
+        print(element)
         date = parse_date(element["C"][0])
 
         if "Ø" in element:
@@ -108,11 +144,12 @@ def _parse_vaccinations_by_region(data) -> "list[VaccinationByRegionRow]":
     parsed_data = []
 
     for element in resp:
-        region = str(element["G0"])
-        count_first = int(element["X"][0]["C"][1])
-        count_second = int(element["X"][1]["C"][1])
-        share_first = float(element["X"][0]["C"][0]) / 100.0
-        share_second = float(element["X"][1]["C"][0]) / 100.0
+        C = element["C"]
+        region = str(C[0])
+        count_first = int(C[3])
+        count_second = int(C[4])
+        share_first = float(C[1]) / 100.0
+        share_second = float(C[2]) / 100.0
 
         parsed_data.append(
             VaccinationByRegionRow(
@@ -237,25 +274,39 @@ def _parse_vaccines_supplied_by_manufacturer_cum(
     return parsed_data
 
 
-def _parse_vaccinations_by_age_group(data) -> "list[VaccinationDose]":
+def _parse_vaccinations_by_age_group(data) -> "list[vaccinations_by_age_group]":
     resp = data["results"][0]["result"]["data"]["dsr"]["DS"][0]["PH"][0]["DM0"]
     parsed_data = []
 
     date = None
-    dose = None
-    r_list = [None, 1]
+    people_vaccinated = None
+    people_fully_vaccinated = None
     for element in resp:
-        date = parse_date(element["G0"])
-        R = R = element["X"][0]["R"] if "R" in element["X"][0] else None
+        C = element["C"]
+        if len(C) == 3:
+            date = parse_date(C[0])
+            people_vaccinated = C[1]
+            people_fully_vaccinated = C[2]
+        elif len(C) == 2:
+            date = parse_date(C[0])
+            R = element["R"]
+            if R == 2:
+                people_fully_vaccinated = C[1]
+            else:
+                people_vaccinated = C[1]
+        elif len(C) == 1:
+            # R == 6
+            date = parse_date(C[0])
+        else:
+            raise Exception("Unknown item length!")
 
-        if R not in r_list:
-            print(R)
-            raise Exception("Unknown R value!")
-
-        if R == None:
-            dose = element["X"][0]["M0"]
-
-        parsed_data.append(VaccinationDose(date=date, dose=dose))
+        parsed_data.append(
+            VaccinationByDayRow(
+                date=date,
+                first_dose=people_vaccinated,
+                second_dose=people_fully_vaccinated,
+            )
+        )
 
     return parsed_data
 
@@ -273,26 +324,26 @@ def _parse_vaccinations_by_region_by_day(data):
     resp = data["results"][0]["result"]["data"]["dsr"]["DS"][0]["PH"][0]["DM0"]
     parsed_data = []
 
-    r_list = [None, 1]
-
     people_vaccinated = None
     people_fully_vaccinated = None
     for element in resp:
-        date = parse_date(element["G0"])
+        C = element["C"]
 
-        R = element["X"][0]["R"] if "R" in element["X"][0] else None
-
-        if R == None:
-            people_vaccinated = element["X"][0]["M0"]
-            people_fully_vaccinated = (
-                element["X"][1]["M0"] if len(element["X"]) > 1 else 0
-            )
-
-        if R == 1:
-            # as far as we know people_vaccinated same as previous
-            people_fully_vaccinated = (
-                element["X"][1]["M0"] if len(element["X"]) > 1 else 0
-            )
+        if len(C) == 3:
+            date = parse_date(C[0])
+            people_vaccinated = C[1]
+            people_fully_vaccinated = C[2]
+        elif len(C) == 2:
+            date = parse_date(C[0])
+            R = element["R"]
+            if R == 2:
+                people_fully_vaccinated = C[1]
+            else:
+                people_vaccinated = C[1]
+        elif len(C) == 1:
+            date = parse_date(C[0])
+        else:
+            raise Exception("Unknown item length!")
 
         parsed_data.append(
             VaccinationByDayRow(
@@ -316,20 +367,41 @@ def _parse_vaccinations_by_municipalities_share(data) -> "list[VaccinationMunSha
     parsed_data = []
 
     for el in resp:
-        name, share1, population, share2 = el["C"]
-        print(name, share1, population, share2, sep="\t")
-        dose1 = round(int(population) * float(share1))
-        dose2 = round(int(population) * float(share2))
-        parsed_data.append(
-            VaccinationMunShare(
-                name=name,
-                dose1=dose1,
-                share1=float(share1),
-                dose2=dose2,
-                share2=float(share2),
-                population=population,
+        C = el["C"]
+        R = el.get("R", None)
+        if len(C) == 6:
+            name, population, share2, share1, dose1, dose2 = el["C"]
+            parsed_data.append(
+                VaccinationMunShare(
+                    name=name,
+                    dose1=dose1,
+                    share1=float(share1),
+                    dose2=dose2,
+                    share2=float(share2),
+                    population=population,
+                )
             )
-        )
+        elif len(C) == 5:
+            if R == 32:
+                name, population, share2, share1, dose1 = el["C"]
+                dose2 = int(population * float(share2))
+                print(dose2)
+                parsed_data.append(
+                    VaccinationMunShare(
+                        name=name,
+                        dose1=dose1,
+                        share1=float(share1),
+                        dose2=dose2,
+                        share2=float(share2),
+                        population=population,
+                    )
+                )
+            else:
+                print(el)
+                raise Exception(f"Unknown R: {R}")
+        else:
+            print(el)
+            raise Exception(f"Unknown C length: {len(C)}")
 
     return parsed_data
 
@@ -369,13 +441,17 @@ def _parse_vaccinations_age_group_by_region_on_day(
                 total_share=float(item[0]),
                 total_count=item[1],
             )
-        raise Exception('Unknown item length!')
+        raise Exception("Unknown item length!")
 
     parsed_data = []
     for el in resp:
-        region = el["G0"]
-        first_dose = parse_resp_data(region, el["X"][0]["C"])
-        second_dose = parse_resp_data(region, el["X"][1]["C"])
+        print(el)
+        C = el["C"]
+        region = C[0]
+        first_dose_data = [C[1], C[2], C[5], C[6]]
+        second_dose_data = [C[3], C[4], C[7], C[8]]
+        first_dose = parse_resp_data(region, first_dose_data)
+        second_dose = parse_resp_data(region, second_dose_data)
         parsed_data.append(
             VaccinationAgeGroupByRegionOnDay(
                 region=region, dose1=first_dose, dose2=second_dose
@@ -412,3 +488,15 @@ def _parse_vaccinations_by_manufacturer_supplied_used(
             raise Exception("Unknown [C] length")
 
     return parsed_data
+
+
+def _parse_vaccinations_gender_by_date(data):
+    if "DS" not in data["results"][0]["result"]["data"]["dsr"]:
+        error = data["results"][0]["result"]["data"]["dsr"]["DataShapes"][0][
+            "odata.error"
+        ]
+        print(error)
+        raise Exception("Something went wrong!")
+
+    resp = data["results"][0]["result"]["data"]["dsr"]["DS"][0]["PH"][0]["DM0"]
+    return resp[0].get("M0", None)
\ No newline at end of file
