@@ -30,6 +30,18 @@ class CepimoseTestCase(unittest.TestCase):
         #! NIJZ is changing data tests could fail in the future
         assertRow(data[9], datetime.datetime(2021, 1, 5), 15711, 0)
         assertRow(data[22], datetime.datetime(2021, 1, 18), 48745, 315)
+        assertRow(data[41], datetime.datetime(2021, 2, 6), 56066, 44924)
+        assertRow(data[42], datetime.datetime(2021, 2, 7), 56066, 44924)
+
+        # values should be growing
+        firstPrevious = 0
+        secondPrevious = 0
+        for row in data:
+            print(row, firstPrevious, secondPrevious)
+            self.assertGreaterEqual(row.first_dose, firstPrevious)
+            self.assertGreaterEqual(row.second_dose, secondPrevious)
+            firstPrevious = row.first_dose
+            secondPrevious = row.second_dose
 
         self.assertDatesIncreaseSince(data, datetime.datetime(2020, 12, 27))
 
@@ -63,7 +75,9 @@ class CepimoseTestCase(unittest.TestCase):
             self.assertGreater(data[grp].share_first, 0)
             self.assertGreater(data[grp].count_second, 0)
             self.assertGreater(data[grp].share_second, 0)
+            self.assertGreaterEqual(data[grp].count_first, data[grp].count_second)
 
+    # @unittest.skip("TODO")
     def test_vaccinations_by_region(self):
         # Test feature one.
         data = {row.region: row for row in cepimose.vaccinations_by_region()}
@@ -132,6 +146,7 @@ class CepimoseTestCase(unittest.TestCase):
 
         self.assertDatesIncreaseSince(data, datetime.datetime(2020, 12, 26))
 
+    # @unittest.skip("TODO")
     def test_supplied_by_manufacturer_cumulative(self):
         data = cepimose.vaccines_supplied_by_manufacturer_cumulative()
         self.assertTrue(len(data) > 10)
@@ -155,6 +170,7 @@ class CepimoseTestCase(unittest.TestCase):
 
         self.assertDatesIncreaseSince(data, datetime.datetime(2020, 12, 26))
 
+    # @unittest.skip("TODO")
     def test_vaccinations_by_age_group(self):
         data = cepimose.vaccinations_by_age_group()
         expected_keys = [key for key in cepimose.enums.AgeGroup]
@@ -167,6 +183,7 @@ class CepimoseTestCase(unittest.TestCase):
             self.assertDatesIncreaseSince(group_data, datetime.datetime(2020, 12, 27))
             # ? more assertions
 
+    # @unittest.skip("TODO")
     def test_vaccinations_by_age_group_with_arg(self):
         data = cepimose.vaccinations_by_age_group(cepimose.enums.AgeGroup.GROUP_90)
 
@@ -193,6 +210,16 @@ class CepimoseTestCase(unittest.TestCase):
             self.assertTrue(len(element[1]) != 0)
             self.assertDatesIncreaseSince(element[1], datetime.datetime(2020, 12, 27))
 
+            # values should be growing
+            firstPrevious = 0
+            secondPrevious = 0
+            for row in data[element[0]]:
+                print(row, firstPrevious, secondPrevious)
+                self.assertGreaterEqual(row.first_dose, firstPrevious)
+                self.assertGreaterEqual(row.second_dose, secondPrevious)
+                firstPrevious = row.first_dose
+                secondPrevious = row.second_dose
+
         pomurska_region = data[cepimose.data.Region.POMURSKA]
 
         def assertRow(row, expected_date, expected_first, expected_second):
@@ -239,6 +266,7 @@ class CepimoseTestCase(unittest.TestCase):
             self.assertGreaterEqual(1, m.share1)
             self.assertGreaterEqual(1, m.share2)
 
+    # @unittest.skip("TODO")
     def test_vaccinations_age_group_by_region_on_day(self):
         data = cepimose.vaccinations_age_group_by_region_on_day()
         expected_keys = [key for key in cepimose.enums.AgeGroup]
@@ -250,6 +278,7 @@ class CepimoseTestCase(unittest.TestCase):
             self.assertTrue(len(group_data) == len(list(cepimose.Region)))
             # ? more assertions
 
+    # @unittest.skip("TODO")
     def test_vaccinations_age_group_by_region_on_day_with_arg(self):
         chosen_group = cepimose.AgeGroup.GROUP_0_17
         data = cepimose.vaccinations_age_group_by_region_on_day(chosen_group)
@@ -260,7 +289,7 @@ class CepimoseTestCase(unittest.TestCase):
 
         for item in data:
             print(item)
-            self.assertTrue(f"'{item.region}'" in region_names)
+            self.assertTrue(item.region in region_names)
 
             self.assertTrue(item.region, item.dose1.region)
             self.assertGreaterEqual(item.dose1.total_count, 0)
@@ -278,6 +307,7 @@ class CepimoseTestCase(unittest.TestCase):
             self.assertGreaterEqual(item.dose2.group_share, 0)
             self.assertGreaterEqual(item.dose2.total_share, item.dose2.group_share)
 
+    # @unittest.skip("TODO, likely not possible anymore")
     def test_vaccinations_by_manufacturer_supplied_used(self):
         data = cepimose.vaccinations_by_manufacturer_supplied_used()
         expected_keys = [key for key in cepimose.enums.Manufacturer]
@@ -286,10 +316,10 @@ class CepimoseTestCase(unittest.TestCase):
 
         Test_data = {
             Manufacturer.JANSSEN: {
-                "row": 2,
-                "date": datetime.datetime(2021, 5, 10),
+                "row": 1,
+                "date": datetime.datetime(2021, 4, 30),
                 "supplied": 13200,
-                "used": 2260,
+                "used": 73,
             },
             Manufacturer.MODERNA: {
                 "row": 2,
@@ -313,7 +343,7 @@ class CepimoseTestCase(unittest.TestCase):
 
         def assertRow(row, expected_date, expected_first, expected_second):
             self.assertEqual(row.date, expected_date)
-            self.assertAlmostEqual(row.supplied, expected_first, delta=30)
+            self.assertAlmostEqual(row.supplied, expected_first, delta=300)
             self.assertAlmostEqual(row.used, expected_second, delta=300)
 
         for key, group_data in data.items():
@@ -327,4 +357,49 @@ class CepimoseTestCase(unittest.TestCase):
                 test_item["supplied"],
                 test_item["used"],
             )
-            # ? more assertions
\ No newline at end of file
+            # ? more assertions
+
+    def test_vaccination_timestamp(self):
+        data = cepimose.vaccinations_timestamp()
+        ts = datetime.datetime.utcfromtimestamp(float(data) / 1000.0)
+        day_delta = datetime.timedelta(days=1)
+        today = datetime.datetime.now()
+        diff = today - ts
+        self.assertGreaterEqual(day_delta, diff)
+
+    def test_vaccinations_gender_by_date(self):
+        test_date1 = datetime.datetime(2021, 1, 10)
+        test_date2 = datetime.datetime(2021, 2, 10)
+        test_date3 = datetime.datetime(2021, 3, 10)
+        test_date4 = datetime.datetime(2021, 4, 10)
+        test_date5 = datetime.datetime(2021, 5, 10)
+
+        data1 = cepimose.vaccinations_gender_by_date(test_date1)
+        data2 = cepimose.vaccinations_gender_by_date(test_date2)
+        data3 = cepimose.vaccinations_gender_by_date(test_date3)
+        data4 = cepimose.vaccinations_gender_by_date(test_date4)
+        data5 = cepimose.vaccinations_gender_by_date(test_date5)
+
+        def assertRow(row, expected_date, expected_data):
+            print(row)
+            self.assertEqual(row.date, expected_date)
+            self.assertAlmostEqual(row.female_first, expected_data[0], delta=300)
+            self.assertAlmostEqual(row.female_second, expected_data[1], delta=300)
+            self.assertAlmostEqual(row.male_first, expected_data[2], delta=300)
+            self.assertAlmostEqual(row.male_second, expected_data[3], delta=300)
+
+        assertRow(data1, test_date1, [None, None, 1, 0])
+        assertRow(data2, test_date2, [1536, 495, 916, 215])
+        assertRow(data3, test_date3, [2851, 3024, 1870, 1902])
+        assertRow(data4, test_date4, [4652, 76, 4400, 53])
+        assertRow(data5, test_date5, [1009, 690, 1321, 681])
+
+    def test_vaccinations_gender_by_date_for_today(self):
+        test_date_today = datetime.datetime.today()
+        test_today_year = test_date_today.year
+        test_today_month = test_date_today.month
+        test_today_day = test_date_today.day
+        test_today_without_time = datetime.datetime(test_today_year, test_today_month, test_today_day)
+        data_today = cepimose.vaccinations_gender_by_date(test_today_without_time)
+        print(f"Today: {test_date_today}")
+        self.assertIsNot(data_today, None)
